#include <stdio.h>
#include <stdlib.h>

/* Robot programs:

   At each step, the robot can either

   * emit a beep

   * turn right or left by some number of degrees (fractional)

   * move forward by some number of centimeters (integer)

   Given a robot program, we'll need to determine how far it will
   move, whether it ever beeps, etc.
*/

/* Like "step_u.c", but instead of using the same
   amont of memory for each step, make steps that
   need extra data use a specifically extended
   struct type. */

typedef enum {
  beep_step,
  turn_left_step,
  turn_right_step,
  forward_step,
} step_kind_t;

typedef struct step_t {
  step_kind_t kind;
  struct step_t *next;
} step_t;

typedef struct {
  step_t base;
  double degrees;
} turn_step_t;

typedef struct {
  step_t base;
  int cm;
} forward_step_t;

/* allocators */

step_t *make_beep() {
  step_t *s = malloc(sizeof(step_t));
  s->kind = beep_step;
  return s;
}

step_t *make_turn(step_kind_t kind, double degrees) {
  turn_step_t *ts = malloc(sizeof(turn_step_t));
  ts->base.kind = kind;
  ts->degrees = degrees;
  return (step_t *)ts;
}

step_t *make_forward(int cm) {
  forward_step_t *fs = malloc(sizeof(forward_step_t));
  fs->base.kind = forward_step;
  fs->cm = cm;
  return (step_t *)fs;
}

/* determining distance robot will move */

int distance(step_t *s) {
  int dist = 0;

  while (s != NULL) {
    switch (s->kind) {
    case forward_step:
      {
	forward_step_t *fs = (forward_step_t *)s;
	dist += fs->cm;
      }
      break;
    default:
      break;
    }
    s = s->next;
  }

  return dist;
}

/* example use */

int main() {
  step_t *l1 = make_turn(turn_left_step, 90);
  step_t *f10 = make_forward(10);
  step_t *r1 = make_turn(turn_right_step, 90);
  step_t *f3 = make_forward(3);
  step_t *b1 = make_beep();

  l1->next = f10;
  f10->next = r1;
  r1->next = f3;
  f3->next = b1;
  b1->next = NULL;

  printf("%d\n", distance(l1));

  return 0;
}
